@@ -14,5 +14,5 @@ def test_data_dsl():
     model = mmD.model_from_file(os.path.join(current_dir,
                                              'models',
                                              'data_structures.edata'))
-    assert(model is not None)
-    assert(len(model.data) == 3)
+    assert model is not None
+    assert len(model.data) == 3
