@@ -1,12 +1,6 @@
-import sys
-import pytest
 import subprocess
 
 
-if (3, 6) <= sys.version_info < (3, 8):
-    pytest.skip("Temporary workaround for Travis problems", allow_module_level=True)
-
-
 def test_subcommand():
     """
     Test that a command from the example project is registered.
