@@ -250,15 +250,14 @@ set namespaceSeparator .
             # If attribute is a reference
             # mult = attr.mult if not attr.mult == MULT_ONE else ""
             if attr.cont:
-                arr = "*--"
+                arr = "*-->"
             else:
-                arr = "o--"
-            if attr.mult == MULT_ZEROORMORE:
-                arr = arr + ' "0..*"'
-            elif attr.mult == MULT_ONEORMORE:
-                arr = arr + ' "1..*"'
-            return '{} {} {}\n'.format(
-                cls._tx_fqn, arr, attr.cls._tx_fqn)
+                arr = "o-->"
+            name = attr.name
+            if attr.mult != "1":
+                name += " " + attr.mult
+            return '{} {} {}: {}\n'.format(
+                cls._tx_fqn, arr, attr.cls._tx_fqn, name)
 
     def render_inherited_by(self, base, special):
         return '{} <|-- {}\n'.format(base._tx_fqn, special._tx_fqn)
@@ -334,56 +333,57 @@ def model_export_to_file(f, model=None, repo=None):
         attrs = ""
         obj_cls = obj.__class__
         name = ""
-        for attr_name, attr in obj_cls._tx_attrs.items():
+        if hasattr(obj_cls, '_tx_attrs'):
+            for attr_name, attr in obj_cls._tx_attrs.items():
 
-            attr_value = getattr(obj, attr_name)
-            if attr_value is None:
-                continue
+                attr_value = getattr(obj, attr_name)
+                if attr_value is None:
+                    continue
 
-            endmark = 'arrowtail=diamond dir=both' if attr.cont else ""
-            required = "+" if attr.mult in \
-                [MULT_ONE, MULT_ONEORMORE] else ""
+                endmark = 'arrowtail=diamond dir=both' if attr.cont else ""
+                required = "+" if attr.mult in \
+                    [MULT_ONE, MULT_ONEORMORE] else ""
 
-            if attr.mult in [MULT_ONEORMORE, MULT_ZEROORMORE]:
-                if all([type(x) in PRIMITIVE_PYTHON_TYPES
-                        for x in attr_value]):
-                    attrs += "{}{}:list=[".format(required, attr_name)
-                    attrs += ",".join([dot_repr(x) for x in attr_value])
-                    attrs += "]\\l"
-                else:
-                    for idx, list_obj in enumerate(attr_value):
-                        if list_obj is not None:
-                            if type(list_obj) in PRIMITIVE_PYTHON_TYPES:
-                                f.write(
-                                    '{} -> "{}:{}" [label="{}:{}" {}]\n'
-                                    .format(id(obj), list_obj,
-                                            type(list_obj).__name__,
-                                            attr_name, idx, endmark))
-                            else:
-                                f.write('{} -> {} [label="{}:{}" {}]\n'
-                                        .format(id(obj), id(list_obj),
-                                                attr_name, idx, endmark))
-                                _export(list_obj)
-            else:
-
-                # Plain attributes
-                if type(attr_value) is text and attr_name != 'name':
-                    attr_value = dot_repr(attr_value)
-
-                if type(attr_value) in PRIMITIVE_PYTHON_TYPES:
-                    if attr_name == 'name':
-                        name = attr_value
+                if attr.mult in [MULT_ONEORMORE, MULT_ZEROORMORE]:
+                    if all([type(x) in PRIMITIVE_PYTHON_TYPES
+                            for x in attr_value]):
+                        attrs += "{}{}:list=[".format(required, attr_name)
+                        attrs += ",".join([dot_repr(x) for x in attr_value])
+                        attrs += "]\\l"
                     else:
-                        attrs += "{}{}:{}={}\\l".format(
-                            required, attr_name, type(attr_value)
-                            .__name__, attr_value)
+                        for idx, list_obj in enumerate(attr_value):
+                            if list_obj is not None:
+                                if type(list_obj) in PRIMITIVE_PYTHON_TYPES:
+                                    f.write(
+                                        '{} -> "{}:{}" [label="{}:{}" {}]\n'
+                                        .format(id(obj), list_obj,
+                                                type(list_obj).__name__,
+                                                attr_name, idx, endmark))
+                                else:
+                                    f.write('{} -> {} [label="{}:{}" {}]\n'
+                                            .format(id(obj), id(list_obj),
+                                                    attr_name, idx, endmark))
+                                    _export(list_obj)
                 else:
-                    # Object references
-                    if attr_value is not None:
-                        f.write('{} -> {} [label="{}" {}]\n'.format(
-                            id(obj), id(attr_value),
-                            attr_name, endmark))
-                        _export(attr_value)
+
+                    # Plain attributes
+                    if type(attr_value) is text and attr_name != 'name':
+                        attr_value = dot_repr(attr_value)
+
+                    if type(attr_value) in PRIMITIVE_PYTHON_TYPES:
+                        if attr_name == 'name':
+                            name = attr_value
+                        else:
+                            attrs += "{}{}:{}={}\\l".format(
+                                required, attr_name, type(attr_value)
+                                .__name__, attr_value)
+                    else:
+                        # Object references
+                        if attr_value is not None:
+                            f.write('{} -> {} [label="{}" {}]\n'.format(
+                                id(obj), id(attr_value),
+                                attr_name, endmark))
+                            _export(attr_value)
 
         name = "{}:{}".format(name, obj_cls.__name__)
 
