@@ -8,13 +8,13 @@ have named this language textX ;)
 """
 from __future__ import unicode_literals
 import re
+import codecs
 from arpeggio import StrMatch, Optional, ZeroOrMore, OneOrMore, Sequence,\
     OrderedChoice, UnorderedGroup, Not, And, RegExMatch, Match, NoMatch, EOF, \
     ParsingExpression, ParserPython, visit_parse_tree
 from arpeggio.export import PMDOTExporter
 from arpeggio import RegExMatch as _
 from textx.scoping.rrel import rrel_expression, RRELVisitor
-from textx.six import string_types
 
 from .exceptions import TextXError, TextXSyntaxError, TextXSemanticError
 from .const import MULT_ONE, MULT_ZEROORMORE, MULT_ONEORMORE, \
@@ -26,6 +26,22 @@ if sys.version < '3':
 else:
     text = str
 
+# Interpreting backslash sequences.
+# See https://stackoverflow.com/a/24519338/2024430
+ESCAPE_SEQUENCE_RE = re.compile(r'''
+    ( \\U........      # 8-digit hex escapes
+    | \\u....          # 4-digit hex escapes
+    | \\x..            # 2-digit hex escapes
+    | \\[0-7]{1,3}     # Octal escapes
+    | \\N\{[^}]+\}     # Unicode characters by name
+    | \\[\\'"abfnrtv]  # Single-character escapes
+    )''', re.UNICODE | re.VERBOSE)
+
+def decode_escapes(s):
+    def decode_match(match):
+        return codecs.decode(match.group(0), 'unicode-escape')
+
+    return ESCAPE_SEQUENCE_RE.sub(decode_match, s)
 
 # textX grammar
 def textx_model():          return (ZeroOrMore(import_or_reference_stm),
@@ -67,7 +83,8 @@ def assignment_rhs():       return [simple_match, reference], Optional(repeat_mo
 # References
 def reference():            return [rule_ref, obj_ref]
 def rule_ref():             return ident
-def obj_ref():              return '[', class_name, Optional('|', obj_ref_rule, Optional('|', rrel_expression)), ']'
+# TODO: Remove "|" optional sep in version 4.0.
+def obj_ref():              return '[', class_name, Optional([':', '|'], obj_ref_rule, Optional('|', rrel_expression)), ']'
 
 def rule_name():            return ident
 def obj_ref_rule():         return ident
@@ -351,20 +368,26 @@ class TextXVisitor(RRELVisitor):
                     # Recursively append all referenced classes.
                     def _add_reffered_classes(rule, inh_by, start=False):
                         if rule.root and not start:
-                            if hasattr(rule, '_tx_class'):
-                                _determine_rule_type(rule._tx_class)
-                                if rule._tx_class._tx_type != RULE_MATCH and\
-                                        rule._tx_class not in inh_by:
-                                    inh_by.append(rule._tx_class)
+                            _determine_rule_type(rule._tx_class)
+                            if rule._tx_class._tx_type != RULE_MATCH and\
+                                    rule._tx_class not in inh_by:
+                                inh_by.append(rule._tx_class)
+                                # stop after first added/found type
+                                return True
                         else:
+                            is_ordered_choice = type(rule) is OrderedChoice
+                            inh_added = False
                             for r in rule.nodes:
-                                _add_reffered_classes(r, inh_by)
+                                inh_added |= _add_reffered_classes(r, inh_by)
+                                if inh_added and not is_ordered_choice:
+                                    # If not ordered choice we should get out
+                                    # early as the rest of the rule shouldn't
+                                    # influence the inheritance hierarchy.
+                                    break
+                            return inh_added
+                        return False
 
-                    if type(rule) is OrderedChoice:
-                        for r in rule.nodes:
-                            _add_reffered_classes(r, cls._tx_inh_by)
-                    else:
-                        _add_reffered_classes(rule, cls._tx_inh_by, start=True)
+                    _add_reffered_classes(rule, cls._tx_inh_by, start=True)
 
         # Multi-pass rule type resolving to support circular rule references.
         # `has_change` is a list to support outer scope variable change in
@@ -584,7 +607,7 @@ class TextXVisitor(RRELVisitor):
                     .format(name,
                             self.grammar_parser.pos_to_linecol(node.position)))
 
-            if name == 'split' and not isinstance(value, string_types):
+            if name == 'split' and not isinstance(value, str):
                 raise TextXError("param split requires a string parameter")
             if name == 'split' and len(value)==0:
                 raise TextXError("param split requires a non-empty string parameter")
@@ -801,7 +824,7 @@ class TextXVisitor(RRELVisitor):
             cls_attr.mult = MULT_OPTIONAL
             base_rule_name = 'BOOL'
 
-            # ?= assigment should have default value of False.
+            # ?= assignment should have default value of False.
             # so we shall mark it as such.
             cls_attr.bool_assignment = True
 
@@ -858,6 +881,9 @@ class TextXVisitor(RRELVisitor):
     def visit_str_match(self, node, children):
         try:
             to_match = children[0][1:-1]
+            if '\\' in to_match:
+                to_match = decode_escapes(to_match)
+
         except IndexError:
             to_match = ''
 
@@ -882,9 +908,7 @@ class TextXVisitor(RRELVisitor):
             regex.compile()
         except Exception as e:
             line, col = self.grammar_parser.pos_to_linecol(node[1].position)
-            raise TextXSyntaxError(
-                "{} at {}"
-                .format(text(e), text((line, col))), line, col)
+            raise TextXSyntaxError(text(e), line, col)
         return regex
 
     def visit_obj_ref(self, node, children):
@@ -898,9 +922,9 @@ class TextXVisitor(RRELVisitor):
                 'Primitive type instances can not be referenced at {}.'
                 .format((line, col)), line, col)
         if len(children) > 1:
-            rule_name = children[1]
-            if len(children) > 2:
-                rrel_tree = children[2]
+            rule_name = children[2]
+            if len(children) > 3:
+                rrel_tree = children[3]
         else:
             # Default rule for matching obj cross-refs
             rule_name = 'ID'
@@ -959,8 +983,10 @@ def language_from_str(language_def, metamodel, file_name):
     try:
         parse_tree = parser.parse(language_def, file_name)
     except NoMatch as e:
-        line, col = parser.pos_to_linecol(e.position)
-        raise TextXSyntaxError(text(e), line, col)
+        e.eval_attrs()
+        raise TextXSyntaxError(e.message, e.line, e.col,
+                               filename=e.parser.file_name,
+                               context=e.context)
 
     # Construct new parser and meta-model based on the given language
     # description.
@@ -975,7 +1001,7 @@ def language_from_str(language_def, metamodel, file_name):
     metamodel._parser_blueprint = lang_parser
 
     if metamodel.debug:
-        # Create dot file for debuging purposes
+        # Create dot file for debugging purposes
         PMDOTExporter().exportFile(
             lang_parser.parser_model,
             "{}_parser_model.dot".format(metamodel.rootcls.__name__))
