@@ -9,7 +9,6 @@ import warnings
 from os.path import join, abspath, dirname
 from collections import OrderedDict
 from arpeggio import DebugPrinter
-from textx.six import add_metaclass
 from textx.lang import language_from_str, python_type, BASE_TYPE_NAMES, ID, \
     BOOL, INT, FLOAT, STRICTFLOAT, STRING, NUMBER, BASETYPE, OBJECT
 from textx.const import MULT_ONE, MULT_ZEROORMORE, MULT_ONEORMORE, \
@@ -18,7 +17,6 @@ from textx.exceptions import TextXError
 from .registration import LanguageDesc, metamodel_for_language
 from .model_params import ModelParams, ModelParamDefinitions
 from textx.scoping.rrel import create_rrel_scope_provider
-from textx.six import string_types
 
 if sys.version < '3':
     text = unicode  # noqa
@@ -138,7 +136,7 @@ class TextXMetaModel(DebugPrinter):
             to use instead of the dynamically created or a callable providing
             those classes. The callable must accept a rule name and return a
             class for that rule name or None.
-        obj_processors(dict): A dict of user supplied object processors keyed
+        _obj_processors(dict): A dict of user supplied object processors keyed
             by rule/class name (may be a fully qualified name).
         rootcls(TextXClass): A language class that is a root of the meta-model.
         root_path(str): The root dir used for the import statement.
@@ -213,11 +211,8 @@ class TextXMetaModel(DebugPrinter):
         # Registered model processors
         self._model_processors = []
 
-        # Registered object processors
-        self.obj_processors = {}
-
         # Match rule and base type conversion callables
-        self.type_convertors = {
+        self._default_obj_processors = {
             'BOOL': lambda x: x == '1' or x.lower() == 'true',
             'INT': lambda x: int(x),
             'FLOAT': lambda x: float(x),
@@ -226,6 +221,9 @@ class TextXMetaModel(DebugPrinter):
                                                 r'"').replace(r"\'", "'"),
         }
 
+        # Registered object processors (use _default_obj_processors)
+        self.register_obj_processors({})
+
         # Registered scope provider
         self.scope_providers = {}
 
@@ -276,7 +274,7 @@ class TextXMetaModel(DebugPrinter):
     def register_scope_providers(self, sp):
         self.scope_providers = sp
         for k, v in self.scope_providers.items():
-            if isinstance(v, string_types):
+            if isinstance(v, str):
                 self.scope_providers[k] = create_rrel_scope_provider(v)
 
     def _namespace_for_file_name(self, file_name):
@@ -364,8 +362,7 @@ class TextXMetaModel(DebugPrinter):
                 return '<textx:{} class at {}>'.format(cls._tx_fqn,
                                                        id(cls))
 
-        @add_metaclass(TextXMetaClass)
-        class TextXClass(object):
+        class TextXClass(object, metaclass=TextXMetaClass):
             """
             Dynamically created class. Each textX rule will result in
             creating one Python class with the type name of the rule.
@@ -498,11 +495,38 @@ class TextXMetaModel(DebugPrinter):
         clazz._tx_attrs[name] = attr
         return attr
 
-    def convert(self, value, _type):
+    def has_obj_processor(self, _type):
+        return _type in self._obj_processors
+
+    def process(self, value, _type, filename, col, line):
         """
+        Process a value with the given type
         Convert instances of textx types and match rules to python types.
+
+        Args:
+            value: the value
+            _type: the type of the value (TextX Class)
+            filename: filename for current object
+            col: col for current object
+            line: line for current object
+        Returns:
+            None or a value used by TextX to replace the object during
+            model creation.
         """
-        return self.type_convertors.get(_type, lambda x: x)(value)
+        try:
+            return self._obj_processors.get(_type, lambda x: x)(value)
+        except Exception as e:
+            from textx.exceptions import TextXError
+            if isinstance(e, TextXError):
+                if e.col is None:
+                    e.col = col
+                if e.line is None:
+                    e.line = line
+                if e.filename is None:
+                    e.filename = filename
+                raise e
+            else:
+                raise
 
     def validate(self):
         """
@@ -708,8 +732,8 @@ class TextXMetaModel(DebugPrinter):
             obj_processors(dict): A dictionary where key=class name,
                 value=callable
         """
-        self.obj_processors = obj_processors
-        self.type_convertors.update(obj_processors)
+        self._obj_processors = dict(self._default_obj_processors)
+        self._obj_processors.update(obj_processors)
 
     @property
     def _tx_model_param_definitions(self):
@@ -726,6 +750,7 @@ class TextXMetaMetaModel(object):
 
     def __init__(self):
         self._metamodel = None
+        self.model_param_defs = ModelParamDefinitions()
 
     @property
     def metamodel(self):
