@@ -7,7 +7,7 @@ import codecs
 import traceback
 from collections import OrderedDict
 from arpeggio import Parser, Sequence, NoMatch, EOF, Terminal
-from textx.exceptions import TextXSyntaxError, TextXSemanticError
+from textx.exceptions import TextXError, TextXSyntaxError, TextXSemanticError
 from textx.const import MULT_OPTIONAL, MULT_ONE, MULT_ONEORMORE, \
     MULT_ZEROORMORE, RULE_ABSTRACT, RULE_MATCH, MULT_ASSIGN_ERROR, \
     UNKNOWN_OBJ_ERROR
@@ -89,9 +89,9 @@ def get_parent_of_type(typ, obj):
 
 def get_children(selector, root, children_first=False, should_follow=lambda obj: True):
     """
-    Returns a list of all model elements of type 'typ' starting from model
-    element 'root'. The search process will follow containment links only.
-    Non-containing references shall not be followed.
+    Returns a list of all model elements that satisfies the given predicate
+    starting from the `root` element. The search process will follow containment
+    links only. Non-containing references shall not be followed.
 
     Args:
         selector(callable): a predicate to decide if the object is of interest.
@@ -185,6 +185,41 @@ def get_location(model_obj):
     return {"line": line, "col": col, "filename": the_model._tx_filename}
 
 
+def textxerror_wrap(obj_processor):
+    """
+    This is a decorator to wrap your object processors
+    if you desire to automatically convert any exception
+    into a TextXError (with file/position information).
+
+    This is useful when non-textx exceptions are used
+    to indicate a problem in a model file:
+
+    ```
+    @textxerror_wrap
+    def date_converter(model_date):
+        from datetime import datetime
+        return datetime(day=model_date.d, month=model_date.m, year=model_date.y)
+    ```
+    Args:
+        obj_processor: an object processor (a callable)
+
+    Returns:
+        the decorated object processor
+    """
+    def wrapper(obj):
+        try:
+            return obj_processor(obj)
+        except Exception as e:
+            if isinstance(e, TextXError):
+                raise
+            else:
+                if hasattr(obj, '_tx_position') and hasattr(obj, '_tx_filename'):
+                    raise TextXError(str(e), **get_location(obj)) from e
+                else:
+                    raise TextXError(str(e)) from e
+    return wrapper
+
+
 class ObjCrossRef(object):
     """
     Used for object cross reference resolving.
@@ -217,15 +252,17 @@ class RefRulePosition(object):
         name(str): A name of the target object.
         ref_pos_start(int): Reference starting position
         ref_pos_end(int): Reference ending position
+        def_file_name(str): Definition's model file name
         def_pos_start(int): Starting position of referenced object
         def_pos_end(int): Ending position of referenced object
     """
 
     def __init__(self, name, ref_pos_start, ref_pos_end,
-                 def_pos_start, def_pos_end):
+                 def_file_name, def_pos_start, def_pos_end):
         self.name = name
         self.ref_pos_start = ref_pos_start
         self.ref_pos_end = ref_pos_end
+        self.def_file_name = def_file_name
         self.def_pos_start = def_pos_start
         self.def_pos_end = def_pos_end
 
@@ -290,10 +327,12 @@ def get_model_parser(top_rule, comments_model, **kwargs):
             try:
                 return self.parser_model.parse(self)
             except NoMatch as e:
-                line, col = e.parser.pos_to_linecol(e.position)
-                raise TextXSyntaxError(message=text(e),
-                                       line=line,
-                                       col=col,
+                e.eval_attrs()
+                raise TextXSyntaxError(message=e.message,
+                                       line=e.line,
+                                       col=e.col,
+                                       filename=e.parser.file_name,
+                                       context=e.context,
                                        expected_rules=e.rules)
 
         def get_model_from_file(self, file_name, encoding, debug,
@@ -459,8 +498,11 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
         """
         Process subtree for match rules.
         """
+        line, col = parser.pos_to_linecol(nt.position)
         if isinstance(nt, Terminal):
-            return metamodel.convert(nt.value, nt.rule_name)
+            return metamodel.process(nt.value, nt.rule_name,
+                                     filename=parser.file_name,
+                                     line=line, col=col)
         else:
             # If RHS of assignment is NonTerminal it is a product of
             # complex match rule. Convert nodes to text and do the join.
@@ -468,24 +510,29 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
                 result = "".join([text(process_match(n)) for n in nt])
             else:
                 result = process_match(nt[0])
-            obj_processor = metamodel.obj_processors.get(nt.rule_name, None)
-            if obj_processor:
-                return obj_processor(result)
-            else:
-                return result
+            return metamodel.process(result, nt.rule_name,
+                                     filename=parser.file_name,
+                                     line=line, col=col)
 
     def process_node(node):
+        line, col = parser.pos_to_linecol(node.position)
         if isinstance(node, Terminal):
             from arpeggio import RegExMatch
             if metamodel.use_regexp_group and \
                     isinstance(node.rule, RegExMatch):
                 if node.rule.regex.groups == 1:
                     value = node.extra_info.group(1)
-                    return metamodel.convert(value, node.rule_name)
+                    return metamodel.process(value, node.rule_name,
+                                             filename=parser.file_name,
+                                             line=line, col=col)
                 else:
-                    return metamodel.convert(node.value, node.rule_name)
+                    return metamodel.process(node.value, node.rule_name,
+                                             filename=parser.file_name,
+                                             line=line, col=col)
             else:
-                return metamodel.convert(node.value, node.rule_name)
+                return metamodel.process(node.value, node.rule_name,
+                                         filename=parser.file_name,
+                                         line=line, col=col)
 
         assert node.rule.root, \
             "Not a root node: {}".format(node.rule.rule_name)
@@ -693,13 +740,18 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
         # enter recursive visit of attributes only, if the class of the
         # object being processed is a meta class of the current meta model
         if model_obj.__class__.__name__ in metamodel:
-            current_metaclass_of_obj = metamodel[model_obj.__class__.__name__]
+            if hasattr(model_obj, '_tx_fqn'):
+                current_metaclass_of_obj = metamodel[model_obj._tx_fqn]
+            else:
+                # fallback (not used - unsure if this case is required...):
+                current_metaclass_of_obj = metamodel[model_obj.__class__.__name__]
+            assert current_metaclass_of_obj is not None
 
             for metaattr in current_metaclass_of_obj._tx_attrs.values():
                 # If attribute is base type or containment reference go down
                 if metaattr.cont:
                     attr = getattr(model_obj, metaattr.name)
-                    if attr:
+                    if attr is not None:
                         if metaattr.mult in many:
                             for idx, obj in enumerate(attr):
                                 if obj is not None:
@@ -714,20 +766,21 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
                             if result is not None:
                                 setattr(model_obj, metaattr.name, result)
 
-            # call obj_proc of the current meta_class if type == RULE_ABSTRACT
             if current_metaclass_of_obj._tx_fqn !=\
                     metaclass_of_grammar_rule._tx_fqn:
-                assert RULE_ABSTRACT == metaclass_of_grammar_rule._tx_type
-                obj_processor_current = metamodel.obj_processors.get(
-                    current_metaclass_of_obj.__name__, None)
-                if obj_processor_current:
-                    return_value_current = obj_processor_current(model_obj)
+                # This can happen if grammar rule is abstract or if model is
+                # modified (e.g. expression reduction)
+                if metamodel.has_obj_processor(current_metaclass_of_obj.__name__):
+                    return_value_current = metamodel.process(
+                        model_obj, current_metaclass_of_obj.__name__,
+                        **get_location(model_obj))
 
         # call obj_proc of rule found in grammar
-        obj_processor_grammar = metamodel.obj_processors.get(
-            metaclass_of_grammar_rule.__name__, None)
-        if obj_processor_grammar:
-            return_value_grammar = obj_processor_grammar(model_obj)
+        if metamodel.has_obj_processor(metaclass_of_grammar_rule.__name__):
+            loc = get_location(model_obj)
+            return_value_grammar = metamodel.process(model_obj,
+                                                     metaclass_of_grammar_rule.__name__,
+                                                     **loc)
 
         # both obj_processors are called, if two different processors
         # are defined for the object metaclass and the grammar metaclass
@@ -779,11 +832,17 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
         if hasattr(model, '_tx_metamodel'):
             assert hasattr(model, '_tx_model_params')
 
+        # Load all imported models (e.g. using importURI)
+        # based on the maker `ModelLoader` found in the
+        # defined scope providers:
         for scope_provider in metamodel.scope_providers.values():
             from textx.scoping import ModelLoader
             if isinstance(scope_provider, ModelLoader):
                 scope_provider.load_models(model, encoding=encoding)
 
+        # Load all imported models based on the maker
+        # `ModelLoader` directly attached to model references
+        # (e.g. in case of RREL expressions defined in the grammar):
         for crossref in parser._crossrefs:
             crossref = crossref[2]
             if crossref.scope_provider is not None:
@@ -843,11 +902,10 @@ def parse_tree_to_objgraph(parser, parse_tree, file_name=None,
 
                     # We have model loaded and all link resolved
                     # So we shall do a depth-first call of object
-                    # processors if any processor is defined.
-                    if m._tx_metamodel.obj_processors:
-                        if parser.debug:
-                            parser.dprint("CALLING OBJECT PROCESSORS")
-                        call_obj_processors(m._tx_metamodel, m)
+                    # processors
+                    if parser.debug:
+                        parser.dprint("CALLING OBJECT PROCESSORS")
+                    call_obj_processors(m._tx_metamodel, m)
 
             except:  # noqa
                 # remove all processed models from (global) repo (if present)
@@ -1029,6 +1087,7 @@ class ReferenceResolver:
                                 ref_pos_start=crossref.position,
                                 ref_pos_end=crossref.position + len(
                                     resolved.name),
+                                def_file_name=get_model(resolved)._tx_filename,
                                 def_pos_start=resolved._tx_position,
                                 def_pos_end=resolved._tx_position_end))
 
